#!/usr/bin/perl

use Test::More qw(no_plan);

use FindBin qw($Bin);
use lib "$Bin/lib",              "$Bin/../lib",           "$Bin/../etc";
use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";
use strict;
#use Opsview::Test;
use Opsview::TestCGI;

my $sth;
my $dbh = Opsview->db_Main;
ok( defined $dbh, "Connect to db" );

my $rc;

my $number_of_hosts = $dbh->selectrow_array("SELECT COUNT(*) FROM hosts");

# Tests on hosts
$rc = post('type=host action=new name="test.altinity" ip="test.altinity" hostgroup_new="bum!fluff" check_command=1');
ok( $rc != 0, "Bad character in hostgroup name" );

$rc = post('type=host action=new name="test.altinity" ip="test.altinity" hostgroup_new=bumfluff check_command=1 icon="LOGO - Opsview"');
ok( $rc == 0, "Added new host" );

$rc = post('type=host action=new name="test.altinity" ip="test.altinity hostgroup_new=bumfluff check_command=1"');
ok( $rc != 0, "Added same host, expected failure" );

$rc = post('type=host action=new ip=testy hostgroup_new=bummer check_command=1');
ok( $rc != 0, "Missing name parameter" );
is( $dbh->selectrow_array("SELECT COUNT(*) FROM hosts"), $number_of_hosts + 1, "Only one row created" );

$rc = post('type=host action=new name=testy hostgroup_new=bum$$ check_command=1');
ok( $rc != 0, "Missing ip parameter" );
is( $dbh->selectrow_array("SELECT COUNT(*) FROM hosts"), $number_of_hosts + 1, "Only one row created" );

$rc = post('type=host action=new name="hostgroup.altinity" hostgroup_new="hostgroup_new" ip=hostgroup icon="LOGO - Turbolinux" notification_period=1 check_command=1');
ok( $rc == 0,                                                                                 "Added new host with new hostgroup" );
ok( $dbh->selectrow_array("SELECT COUNT(*) FROM hostgroups WHERE name='hostgroup_new'") == 1, "Added hostgroup" );
my $hostid = $dbh->selectrow_array("SELECT id FROM hosts WHERE name='hostgroup.altinity'");

$rc = post('type=host action=new name="hostgroup2.altinity" hostgroup_new="hostgroup_new2" ip=hostgroup2 check_command=2 icon="LOGO - Apple"');
ok( $rc == 0,                                                                                  "Added another new host with new hostgroup" );
ok( $dbh->selectrow_array("SELECT COUNT(*) FROM hostgroups WHERE name='hostgroup_new2'") == 1, "Added hostgroup2" );
my $hgid = $dbh->selectrow_array("SELECT id FROM hostgroups WHERE name='hostgroup_new2'");

my $host2id = $dbh->selectrow_array("SELECT id FROM hosts WHERE name='hostgroup2.altinity'");

my $oldhgid = $dbh->selectrow_array("SELECT hostgroup FROM hosts WHERE id=$host2id");
$rc = post("type=host action=edit id=$host2id hostgroup=$hgid");
ok( $rc == 0,                                                                        "Edit hostgroup2.altinity to use hostgroup=$hgid" );
ok( $dbh->selectrow_array("SELECT hostgroup FROM hosts WHERE id=$host2id") == $hgid, "Updated in DB" );

$rc = post("type=host action=edit id=$hostid hostgroup=$oldhgid hostgroup=");
ok( $rc == 0,                                                                          "Check blank hostgroup field" );
ok( $dbh->selectrow_array("SELECT hostgroup FROM hosts WHERE id=$hostid") == $oldhgid, "Updated in DB" );

my $old = $dbh->selectrow_array("SELECT icon FROM hosts WHERE id=$hostid");
$rc = post("type=host action=edit id=$hostid icon=rubbish");
isnt( $rc, 0, "Adding incorrect icon" );
is( $dbh->selectrow_array("SELECT icon FROM hosts WHERE id=$hostid"), $old, "No change in DB" );

$rc = post("type=host action=edit id=$hostid icon='LOGO - BEOS'");
is( $rc,                                                              0,             "Edited icon" );
is( $dbh->selectrow_array("SELECT icon FROM hosts WHERE id=$hostid"), "LOGO - BEOS", "DB updated" );

$old = $dbh->selectrow_array("SELECT notification_period FROM hosts WHERE id=$hostid");
$rc  = post("type=host action=edit id=$hostid notification_period=rubbish");
isnt( $rc, 0, "Adding incorrect notification_period" );
is( $dbh->selectrow_array("SELECT notification_period FROM hosts WHERE id=$hostid"), $old, "No change in DB" );

$rc = post("type=host action=edit id=$hostid notification_period=2");
is( $rc,                                                                             0, "Edited notification_period" );
is( $dbh->selectrow_array("SELECT notification_period FROM hosts WHERE id=$hostid"), 2, "DB updated" );

$rc = post("type=host action=edit id=$hostid notification_options=u notification_options=d");
is( $rc,                                                                              0,     "Edited notification_options" );
is( $dbh->selectrow_array("SELECT notification_options FROM hosts WHERE id=$hostid"), "u,d", "DB updated" );

$rc = post("type=host action=edit id=$hostid notification_options=d notification_options=r");
is( $rc,                                                                              0,     "Edited notification_options" );
is( $dbh->selectrow_array("SELECT notification_options FROM hosts WHERE id=$hostid"), "d,r", "DB updated" );

$old = $dbh->selectrow_array("SELECT check_command FROM hosts WHERE id=$hostid");
$rc  = post("type=host action=edit id=$hostid check_command=rubbish");
isnt( $rc, 0, "Adding incorrect check_command" );
is( $dbh->selectrow_array("SELECT check_command FROM hosts WHERE id=$hostid"), $old, "No change in DB" );

$rc = post("type=host action=edit id=$hostid check_command=3");
is( $rc,                                                                       0, "Amended check_command" );
is( $dbh->selectrow_array("SELECT check_command FROM hosts WHERE id=$hostid"), 3, "Updated in DB" );

$rc = post("type=hosttemplate action=new name=host_test$$");
ok( $rc == 0, "Added hosttemplate to use" );

my $htid = $dbh->selectrow_array("SELECT id FROM hosttemplates WHERE name='host_test$$'");
$rc = post("type=host action=edit id=$hostid hosttemplates=$htid");
ok( $rc == 0, "Amended hosttemplate successfully to $htid" );
is( $dbh->selectrow_array("SELECT count(*) FROM hosthosttemplates WHERE hostid=$hostid"), 1, "Changed in DB" );

# Test deletion of host with parent references does not delete the host itself
my $parentid = $dbh->selectrow_array("SELECT id FROM hosts WHERE name='cisco'");
my $childid  = $dbh->selectrow_array("SELECT id FROM hosts WHERE name='test.altinity'");

$dbh->do("DELETE FROM parents");
ok( $dbh->selectrow_array("SELECT COUNT(*) FROM parents") == 0, "No parents set" );
$rc = post("type=host action=edit id=$hostid parents=$parentid parents=$host2id");
ok( $rc == 0,                                                                        "Added parent" );
ok( $dbh->selectrow_array("SELECT COUNT(*) FROM parents WHERE hostid=$hostid") == 2, "Set in parents table" );

$rc = post("type=host action=edit id=$childid parents=$hostid");
ok( $rc == 0,                                                                          "Added child" );
ok( $dbh->selectrow_array("SELECT COUNT(*) FROM parents WHERE parentid=$hostid") == 1, "Set in parents table" );

my $scid1 = $dbh->selectrow_array("SELECT MIN(id) FROM servicechecks");
my $scid2 = $dbh->selectrow_array("SELECT MAX(id) FROM servicechecks");

#
# Exceptions testing
#
cmp_ok( $dbh->selectrow_array("SELECT COUNT(*) FROM servicecheckhostexceptions WHERE id = $hostid"), '==', 0, "No exceptions for host found" );

# Need to specify servicechecks parameter too, otherwise backend will ignore an exception on its own
$rc = post("type=host action=edit hosttemplates=1 id=$hostid servicechecks=${scid1} exception_${scid1}_add='-H earwax'");
cmp_ok( $rc,                                                                                             "==", 0,           "Edit host and added exception" );
cmp_ok( $dbh->selectrow_array("SELECT COUNT(*) FROM servicecheckhostexceptions WHERE host=$hostid"),     '==', 1,           "Only 1 exception for host" );
cmp_ok( $dbh->selectrow_array("SELECT servicecheck FROM servicecheckhostexceptions WHERE host=$hostid"), "==", $scid1,      "Is correct servicecheck" );
cmp_ok( $dbh->selectrow_array("SELECT args FROM servicecheckhostexceptions WHERE host=$hostid"),         "eq", "-H earwax", "Stored exception correctly" );

$rc = post("type=host action=edit id=$hostid servicechecks=${scid2} exception_${scid2}_add=");

# (mst) this was ...add='', but I can't see that making sense

cmp_ok( $rc,                                                                                             "==", 0,      "Edit host and added new exception" );
cmp_ok( $dbh->selectrow_array("SELECT COUNT(*) FROM servicecheckhostexceptions WHERE host=$hostid"),     '==', 1,      "Still only 1 exception for host" );
cmp_ok( $dbh->selectrow_array("SELECT servicecheck FROM servicecheckhostexceptions WHERE host=$hostid"), "==", $scid2, "Is correct servicecheck" );
cmp_ok( $dbh->selectrow_array("SELECT args FROM servicecheckhostexceptions WHERE host=$hostid"),         "eq", "",     "Stored exception correctly (as blank)" );

$rc = post("type=host action=edit id=$hostid servicechecks=$scid2 servicechecks=$scid1 exception_${scid2}_add='-H lobes' exception_${scid1}_add='-s ignited'");
cmp_ok( $rc,                                                                                                             "==", 0,            "Edit double exception!" );
cmp_ok( $dbh->selectrow_array("SELECT COUNT(*) FROM servicecheckhostexceptions WHERE host=$hostid"),                     '==', 2,            "Two now!" );
cmp_ok( $dbh->selectrow_array("SELECT args FROM servicecheckhostexceptions WHERE host=$hostid AND servicecheck=$scid1"), "eq", "-s ignited", "Stored exception correctly for $scid1" );
cmp_ok( $dbh->selectrow_array("SELECT args FROM servicecheckhostexceptions WHERE host=$hostid AND servicecheck=$scid2"), "eq", "-H lobes",   "Stored exception correctly for $scid2" );

$rc = post("type=host action=delete id=$hostid");
ok( $rc == 0,                                                                  "Deleting host: hostgroup.altinity" );
ok( $dbh->selectrow_array("SELECT COUNT(*) FROM hosts WHERE id=$hostid") == 0, "Deleted from DB" );
cmp_ok( $dbh->selectrow_array("SELECT COUNT(*) FROM servicecheckhostexceptions WHERE host = $hostid"), "==", 0, "Associated exceptions deleted too" );

ok( $dbh->selectrow_array("SELECT COUNT(*) FROM parents WHERE hostid=$hostid OR parentid=$hostid") == 0, "Deleted references in parent table" );
ok( $dbh->selectrow_array("SELECT COUNT(*) FROM hosts WHERE id=$parentid OR id=$childid") == 2,          "Referenced hosts not deleted" );

$rc = post("type=host action=new name=host$$ ip=test$$ hostgroup_new=weyhey$$ hosttemplates=1 servicechecks=1 servicechecks=3 check_command=1 icon='LOGO - Opsview'");
ok( $rc == 0, "Created host with custom monitors" );

$hostid = $dbh->selectrow_array("SELECT id FROM hosts WHERE name='host$$'");
my $cols = $dbh->selectcol_arrayref("SELECT servicecheckid FROM hostservicechecks WHERE hostid=$hostid ORDER BY servicecheckid");
ok( join( ",", @$cols ) eq "1,3", "servicechecks added correctly" );

$rc = post("type=host action=edit id=$hostid servicechecks=1 servicechecks=4");
ok( $rc == 0, "Edit custom check host with just servicechecks" );

ok( $dbh->selectrow_array("SELECT servicecheckid FROM hostservicechecks WHERE hostid=$hostid") == 1,             "Still no servicechecks in custom table" );

$rc = post("type=host action=edit id=$hostid hosttemplates=$htid");
ok( $rc == 0, "Edit host to use a template" );

ok( $dbh->selectrow_array("SELECT COUNT(*) FROM hostservicechecks WHERE hostid=$hostid") == 0,       "Servicechecks from custom table" );

$rc = post("type=host action=edit id=$hostid servicechecks=1 servicechecks=4");
ok( $rc == 0, "Edit host with just servicechecks" );

is( $dbh->selectrow_array("SELECT COUNT(*) FROM hostservicechecks WHERE hostid=$hostid"),       2, "Servicechecks in custom table" );

$rc = post("type=host action=new name=hosta$$ ip=testa$$ hostgroup_new=newone hosttemplates=$htid servicechecks=1 servicechecks=2 check_command=3 icon='LOGO - Apple'");
ok( $rc == 0, "Created host with hosttemplate but additional servicechecks" );
$hostid = $dbh->selectrow_array("SELECT id FROM hosts WHERE name='hosta$$'");

is( $dbh->selectrow_array("SELECT COUNT(*) FROM hostservicechecks WHERE hostid=$hostid"),       2, "Servicechecks added into custom table" );

$rc = post("type=hosttemplate action=delete id=$htid");
ok( $rc == 0,                                                                  "hosttemplate deleted" );
ok( $dbh->selectrow_array("SELECT COUNT(*) FROM hosts WHERE id=$hostid") == 1, "Host not deleted" );

$rc = post('type=host action=new name="ipv6" hostgroup_new="hostgroup_ipv6" ip=::1 check_command=2 icon="LOGO - Apple"');
ok( $rc == 0,                                                                                  "Added a host with ipv6 IP address" );
ok( $dbh->selectrow_array("SELECT COUNT(*) FROM hostgroups WHERE name='hostgroup_ipv6'") == 1, "Added hostgroup_ipv6" );
