#!/usr/bin/perl
# Test rest for roles

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;
use Test::LongString;

use Opsview::Test qw(opsview);
use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);

use lib "$Bin/lib";
use Opsview::TestUtils;

use Data::Dump qw(dump);

my $content;
my $ua = Test::WWW::Mechanize::Catalyst->new( stack_depth => 0 );

rest_log_in( $ua, "admin", "initial" );

$ua->get_ok( "http://localhost/rest/config/role?rows=5&page=1&cols=contacts,name,id,description,accesses", "Only get 1st page of roles" );
my $res_data = eval $ua->content;
my $expected =
{
          'list' => [
                      {
                        'contacts' => [],
                        'ref' => '/rest/config/role/1',
                        'name' => 'Public',
                        'id' => '1',
                        'description' => 'Access available for public users',
                        'accesses' => [
                                        {
                                          'ref' => '/rest/config/access/10',
                                          'name' => 'VIEWPORTACCESS'
                                        }
                                      ]
                      },
                      {
                        'contacts' => [],
                        'ref' => '/rest/config/role/2',
                        'name' => 'Authenticated user',
                        'id' => '2',
                        'description' => 'All authenticated users inherit this role',
                        'accesses' => [
                                        {
                                          'ref' => '/rest/config/access/11',
                                          'name' => 'RRDGRAPHS'
                                        }
                                      ]
                      },
                      {
                        'contacts' => [
                                        {
                                          'ref' => '/rest/config/contact/1',
                                          'name' => 'admin'
                                        }
                                      ],
                        'ref' => '/rest/config/role/10',
                        'name' => 'Admin',
                        'id' => '10',
                        'description' => 'Administrator access',
                        'accesses' => [
                                        {
                                          'ref' => '/rest/config/access/3',
                                          'name' => 'ACTIONALL'
                                        },
                                        {
                                          'ref' => '/rest/config/access/9',
                                          'name' => 'ADMINACCESS'
                                        },
                                        {
                                          'ref' => '/rest/config/access/7',
                                          'name' => 'CONFIGUREHOSTS'
                                        },
                                        {
                                          'ref' => '/rest/config/access/15',
                                          'name' => 'CONFIGUREKEYWORDS'
                                        },
                                        {
                                          'ref' => '/rest/config/access/13',
                                          'name' => 'CONFIGURESAVE'
                                        },
                                        {
                                          'ref' => '/rest/config/access/12',
                                          'name' => 'CONFIGUREVIEW'
                                        },
                                        {
                                          'ref' => '/rest/config/access/16',
                                          'name' => 'DOWNTIMEALL'
                                        },
                                        {
                                          'ref' => '/rest/config/access/6',
                                          'name' => 'NOTIFYSOME'
                                        },
                                        {
                                          'ref' => '/rest/config/access/14',
                                          'name' => 'PASSWORDSAVE'
                                        },
                                        {
                                          'ref' => '/rest/config/access/8',
                                          'name' => 'RELOADACCESS'
                                        },
                                        {
                                          'ref' => '/rest/config/access/1',
                                          'name' => 'VIEWALL'
                                        },
                                      ]
                      },
                      {
                        'contacts' => [
                                        {
                                          'ref' => '/rest/config/contact/6',
                                          'name' => 'testviewallchangesome'
                                        }
                                      ],
                        'ref' => '/rest/config/role/11',
                        'name' => 'View all, change some',
                        'id' => '11',
                        'description' => 'Operator',
                        'accesses' => [
                                        {
                                          'ref' => '/rest/config/access/4',
                                          'name' => 'ACTIONSOME'
                                        },
                                        {
                                          'ref' => '/rest/config/access/17',
                                          'name' => 'DOWNTIMESOME'
                                        },
                                        {
                                          'ref' => '/rest/config/access/6',
                                          'name' => 'NOTIFYSOME'
                                        },
                                        {
                                          'ref' => '/rest/config/access/14',
                                          'name' => 'PASSWORDSAVE'
                                        },
                                        {
                                          'ref' => '/rest/config/access/1',
                                          'name' => 'VIEWALL'
                                        },
                                      ]
                      },
                      {
                        'contacts' => [
                                        {
                                          'ref' => '/rest/config/contact/2',
                                          'name' => 'nonadmin'
                                        },
                                        {
                                          'ref' => '/rest/config/contact/5',
                                          'name' => 'onlyunknowns'
                                        },
                                      ],
                        'ref' => '/rest/config/role/12',
                        'name' => 'View some, change some',
                        'id' => '12',
                        'description' => 'Restricted operator',
                        'accesses' => [
                                        {
                                          'ref' => '/rest/config/access/4',
                                          'name' => 'ACTIONSOME'
                                        },
                                        {
                                          'ref' => '/rest/config/access/7',
                                          'name' => 'CONFIGUREHOSTS'
                                        },
                                        {
                                          'ref' => '/rest/config/access/13',
                                          'name' => 'CONFIGURESAVE'
                                        },
                                        {
                                          'ref' => '/rest/config/access/17',
                                          'name' => 'DOWNTIMESOME'
                                        },
                                        {
                                          'ref' => '/rest/config/access/6',
                                          'name' => 'NOTIFYSOME'
                                        },
                                        {
                                          'ref' => '/rest/config/access/14',
                                          'name' => 'PASSWORDSAVE'
                                        },
                                        {
                                          'ref' => '/rest/config/access/2',
                                          'name' => 'VIEWSOME'
                                        },
                                      ]
                      }
                    ],
                     summary => { allrows => 14, page => 1, rows => 5, totalpages => 3, totalrows => 14 },
        };
is_deeply( $res_data, $expected, "Got expected data for host list" ) || diag(Data::Dump::dump($res_data));

$ua->get_ok( "http://localhost/rest/config/role/12", "Get view some, change some role info" );
$expected = {
          'object' => {
                        access_hostgroups => [],
                        access_keywords => [],
                        access_servicegroups => [],
                        all_hostgroups => 1,
                        all_keywords => 0,
                        all_servicegroups => 1,
                        'contacts' => [
                                        {
                                          'ref' => '/rest/config/contact/2',
                                          'name' => 'nonadmin'
                                        },
                                        {
                                          'ref' => '/rest/config/contact/5',
                                          'name' => 'onlyunknowns'
                                        },
                                      ],
                        'monitoringservers' => [],
                        'hostgroups' => [
                                          {
                                            'ref' => '/rest/config/hostgroup/7',
                                            'name' => 'singlehost'
                                          },
                                          {
                                            'ref' => '/rest/config/hostgroup/6',
                                            'name' => 'UK2'
                                          }
                                        ],
                        'name' => 'View some, change some',
                        'id' => '12',
                        'description' => 'Restricted operator',
                        uncommitted => 0,
                        'accesses' => [
                                        {
                                          'ref' => '/rest/config/access/4',
                                          'name' => 'ACTIONSOME'
                                        },
                                        {
                                          'ref' => '/rest/config/access/7',
                                          'name' => 'CONFIGUREHOSTS'
                                        },
                                        {
                                          'ref' => '/rest/config/access/13',
                                          'name' => 'CONFIGURESAVE'
                                        },
                                        {
                                          'ref' => '/rest/config/access/17',
                                          'name' => 'DOWNTIMESOME'
                                        },
                                        {
                                          'ref' => '/rest/config/access/6',
                                          'name' => 'NOTIFYSOME'
                                        },
                                        {
                                          'ref' => '/rest/config/access/14',
                                          'name' => 'PASSWORDSAVE'
                                        },
                                        {
                                          'ref' => '/rest/config/access/2',
                                          'name' => 'VIEWSOME'
                                        },
                                      ],
                      }
        };
$res_data = eval $ua->content;
is_deeply( $res_data, $expected, "Got hash data for role as expected" ) || diag(Data::Dump::dump($res_data));

$ua->put( "http://localhost/rest/config/role/12", content => "{ accesses => [
                                        {
                                          'ref' => '/rest/config/access/4',
                                          'name' => 'ACTIONSOME'
                                        },
                                        {
                                          'name' => 'VIEWSOME'
                                        },
                                        ] 
}" );
is( $ua->status, 200, "PUT OK" );
$res_data                       = eval $ua->content;
$expected->{object}->{accesses} = [ 
    { ref => "/rest/config/access/4", name => "ACTIONSOME" }, 
    { ref => "/rest/config/access/2", name => "VIEWSOME" }, 
];
$expected->{object}->{uncommitted} = 1;
is_deeply( $res_data, $expected, "Got hash data for contact after a PUT to update" );

# Change list of contacts - fail due to contacts where we don't know where they should go
$expected->{object}->{contacts} = [ { name => "demo" }, { name => "nonadmin" } ];
$ua->put( "http://localhost/rest/config/role/12", content => Data::Dump::dump( $expected->{object} ) );
is( $ua->status, 400, "PUT with a lot of data OK" );
$res_data = eval $ua->content;
is( $res_data->{message}, "Error trying to synchronise object" );
like( $res_data->{detail}, qr/^DBIx::Class::Schema::txn_do\(\): There are some contacts which do not have any roles after this operation: onlyunknowns at/ );

# Change list of contacts
$expected->{object}->{contacts} = [ { 
    ref => "/rest/config/contact/13", 
    name => "demo" 
    },{
    ref => "/rest/config/contact/2", 
    name => "nonadmin" 
    },{
    ref => "/rest/config/contact/5", 
    name => "onlyunknowns" 
    },{
    ref => "/rest/config/contact/4", 
    name => "somehosts" 
} ];
$ua->put( "http://localhost/rest/config/role/12", content => Data::Dump::dump( $expected->{object} ) );
is( $ua->status, 200, "PUT with a lot of data OK" );
$res_data = eval $ua->content;
is_deeply( $res_data, $expected, "Got hash data back with expected data" );

$ua->put( "http://localhost/rest/config/role/12", content => '{ name => "bad%char^s!" }' );
is( $ua->status, 400, "PUT failed" );
$res_data = eval $ua->content;
$expected = { message => "Error trying to synchronise object", };
is( $res_data->{message}, $expected->{message}, "Got right error message" );
like( $res_data->{detail}, qr/Failed constraint on name for /, "Got invalid error message" );

# Need to set Content-Type otherwise WWW::Mechanize will assume post sends form data
$ua->add_header( 'Content-type' => "text/x-data-dumper" );
my $creation = {
    name              => "NewRole",
    accesses => [ { name => "VIEWALL" }, { name => "PASSWORDSAVE" } ],
    };
$expected = {
    name => "NewRole",
    id => 22,
    contacts => [],
    monitoringservers => [],
    hostgroups => [],
    accesses => [ 
    { name => "PASSWORDSAVE",
        ref => "/rest/config/access/14", },
    { name => "VIEWALL",
        ref => "/rest/config/access/1", },
    ],
    access_hostgroups => [],
    access_servicegroups => [],
    access_keywords => [],
    all_hostgroups => 0,
    all_servicegroups => 0,
    all_keywords => 0,
    description => undef,
    uncommitted => 1,
};
$ua->post( "http://localhost/rest/config/role", Content => Data::Dump::dump($creation) );
is( $ua->status, 200, "Create succeeded" );
$res_data = eval $ua->content;
is_deeply( $res_data->{object}, $expected, "Expected return of data too" ) || diag("Full dump: ".Data::Dump::dump($res_data->{object}));

$ua->request( HTTP::Request::Common::DELETE("http://localhost/rest/config/role/18") );
is( $ua->status, 200, "Delete succeeded" );

note("Try to get role 18 after delete");
$ua->get("http://localhost/rest/config/role/18");
is( $ua->status, 404, "No object - deleted!" );
$ua->content_contains( "No object found with id=18", "Got error message" );

$ua->request( HTTP::Request::Common::DELETE("http://localhost/rest/config/role/10") );
is( $ua->status, 400, "Delete failed" );
$res_data = eval $ua->content;
is( $res_data->{message}, "Error when deleting object", "Got correct error message" ) || diag( "Full dump: ".Data::Dump::dump($ua->content) );
like( $res_data->{detail}, qr/Cannot delete or update a parent row: a foreign key constraint fails/, "Got expected detail message re: constraint failure" );

1;
