#!/usr/bin/perl
# Test rest

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;
use Test::LongString;

use Opsview::Test qw(nodb);
use Test::WWW::Mechanize;
#use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);

use Opsview::Web;
use Data::Dump qw(dump);

use Opsview::Config;

my $easyxdm_version = "2.4.15.118";

my $content;
my $expected;
#my $ua   = Test::WWW::Mechanize::Catalyst->new( stack_depth => 0 );
#my $json = Test::WWW::Mechanize::Catalyst->new( stack_depth => 0 );
my $ua   = Test::WWW::Mechanize->new( stack_depth => 0 );
$ua->add_header( 'content-type' => 'text/x-data-dumper' );
my $json = Test::WWW::Mechanize->new( stack_depth => 0 );
$json->add_header( 'content-type' => "application/json" );

# Used for testing JSONP
my $js   = Test::WWW::Mechanize->new( stack_depth => 0 );


# Bit rubbish, but couldn't get YAML to read opsview_web.yml directly
#use YAML qw(LoadFile);
#my $web_config = LoadFile("$Bin/../opsview_web.yml");
my $opsview_version = Opsview::Web->config->{version};
my $opsview_build = Opsview::Web->config->{build};

is( $opsview_build, Opsview::Config->opsview_version, "Check that the quick method of getting version number from Opsview::Config also works");

use version;
my $api_version = version->declare( $opsview_version )->numify;
my $api_min_version = "2.0";

my $url_prefix = "http://localhost:3000";

$ua->get_ok("$url_prefix/rest");
is( $ua->status, 200 );
$content = eval $ua->content;
note("Response: \n===\n".$ua->content."\n===\n");
is( $content->{api_version}, $api_version );
is( $content->{api_min_version}, $api_min_version );

cmp_ok( $content->{api_version}, ">=", $content->{api_min_version}, "Confirm that api_min_version is less than or equal to api_version" );

$json->get_ok("$url_prefix/rest");
my $raw_json = $json->content;

$js->get_ok("$url_prefix/rest?callback=callback&content-type=text/javascript", "Check callback without text/javascript header");
is( $js->content, "callback($raw_json);", "Got same content, wrapped in a callback" );

$js->add_header( 'content-type' => "text/javascript" );
$js->get_ok("$url_prefix/rest");
is( $js->content, $raw_json, "Got same content as application/json" );

$js->get_ok("$url_prefix/rest?callback=callback");
is( $js->content, "callback($raw_json);", "Got same content, wrapped in a callback" );

$js->get_ok("$url_prefix/rest?callback=Ext.data.JsonP.callback1");
is( $js->content, "Ext.data.JsonP.callback1($raw_json);", "Got same content, wrapped in a callback with periods in callback name" );

$js->get_ok("$url_prefix/rest?windowname=1");
is( $js->content, qq%<html>
<script type="text/javascript">
window.name='{"easyxdm_version":"$easyxdm_version","api_version":"$api_version","api_min_version":"$api_min_version"}';
</script>
</html>
%, "Got expected data" );


$ua->post( "$url_prefix/rest/login", Content => qq%{ username => "admin", password => "initial" }%, "Login to API2" );
is( $ua->status, 200 );
$ua->content_contains( "token", "Should return some ticket information" );
$content = $ua->content;
like( $content, qr%^\{'token' => '[0-9a-f]{40}'\}$%, "Got expected content" );
my $data = eval $content;
like( $data->{token}, qr/^[0-9a-f]{40}$/, "Got token value" );

$ua->add_header( "x-opsview-username" => "admin" );
$ua->add_header( "x-opsview-token"    => $data->{token} );

$ua->get_ok("$url_prefix/rest/info");
is( $ua->status, 200 );
$content = eval $ua->content;
note("Response: \n===\n".$ua->content."\n===\n");
is( $content->{opsview_version}, $opsview_version );
is( $content->{opsview_build}, $opsview_build );
is( $content->{uuid}, "TestDB" );
like( $content->{opsview_edition}, qr/^community|enterprise$/ );

