#!/usr/bin/perl
# Test rest acls

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib", "$Bin/lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;
use Test::LongString;

use Opsview::Test qw(opsview);
use Opsview::TestUtils;
#use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);
use Test::WWW::Mechanize;

use Data::Dump qw(dump);

my $content;
#my $ua = Test::WWW::Mechanize::Catalyst->new( stack_depth => 0 );
my $ua = Test::WWW::Mechanize->new( stack_depth => 0 );
$ua->add_header( 'content-type' => 'text/x-data-dumper' );
my $readonly = Test::WWW::Mechanize->new( stack_depth => 0 );
$readonly->add_header( 'content-type' => 'text/x-data-dumper' );

$ua->post("http://localhost:3000/rest/login", Content => qq%{ username => "somehosts", password => "somehosts" }%, "Login to API2 as somehosts" );
is( $ua->status, 200 );
$content = $ua->content;
like( $content, qr/token/, "Got token" );
my $data = eval $content;
like($data->{token}, qr/^[0-9a-f]{40}$/, "Got token value" );

$ua->add_header("X-Opsview-Username" => "somehosts");
$ua->add_header("X-Opsview-Token" => $data->{token});

$ua->get_ok("/rest/info", "Get api information" );
$content = $ua->content;
like( $content, qr/'opsview_edition' => '(community|enterprise)'/, "API info" );

$ua->get("/rest/config/contact");
is( $ua->status, 403, "Unauthorised" );
$ua->content_contains( "Access denied", "Got error message" );

$ua->get("/rest/config/host?rows=0");
is( $ua->status, 200, "Authorised for hosts" );
$data = eval $ua->content;
is( $data->{summary}->{allrows}, 5, "Got a subset of hosts due to security" ) || diag("Full dump:".$ua->content);

$ua->get_ok( "/rest/config/host?s.hostgroup.matpath=/&rows=0" );
is( $ua->status, 200, "Try to get a larger list" );
$data = eval $ua->content;
is( $data->{summary}->{allrows}, 5, "Still only 5 - not hackable" );

$ua->post("http://localhost:3000/rest/config/contact", Content => qq%{ name => "blocked" }% );
is( $ua->status, 403, "Contact POST fails due to auth" );

$readonly->post("http://localhost:3000/rest/login", Content => qq%{ username => "viewsomechangenone", password => "viewsomechangenone" }%, "Login to API2 as viewsomechangenone" );
is( $readonly->status, 200 );
$content = $readonly->content;
like( $content, qr/token/, "Got token" );
$data = eval $content;
like($data->{token}, qr/^[0-9a-f]{40}$/, "Got token value" );

$readonly->add_header("X-Opsview-Username" => "viewsomechangenone");
$readonly->add_header("X-Opsview-Token" => $data->{token});
$readonly->get_ok("/rest/info", "Get api information" );

$readonly->get("/rest/config/host");
is( $readonly->status, 403, "Unauthorised for hosts" );
$readonly->content_contains( "Access denied", "Got error message" );

$readonly->get("/rest/reload");
is( $readonly->status, 403, "Unauthorised for reload data" );

$readonly->get("/rest/downtime");
is( $readonly->status, 200, "Can view downtime, due to VIEWALL");

$readonly->post("/rest/downtime", Content => qq%{ name => "blocked" }% );
is( $ua->status, 403, "Downtime POST fails due to auth" );



my $demo = Test::WWW::Mechanize->new( stack_depth => 0 );
rest_log_in( $demo, "demo", "demo", "http://localhost:3000" );

$demo->get("/rest/downtime");
is( $demo->status, 403, "No access to list downtimes as not VIEW access");

