#!/usr/bin/perl
# Test rest

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;
use Test::LongString;

use Opsview::Test qw(opsview);
use Test::WWW::Mechanize;

use Data::Dump qw(dump);

my $content;
my $expected;
my $gwt = Test::WWW::Mechanize->new( stack_depth => 0 );

# Add headers like GWT
$gwt->add_header( 'Accept' => "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8" );
$gwt->add_header( 'Content-type' => "application/x-www-form-urlencoded" );

my $url_prefix="http://localhost:3000";

$gwt->get_ok("$url_prefix/rest");
is( $gwt->status, 200 );
# Don't do a content test - this is done in t/650-api2-version.t

$gwt->post("$url_prefix/rest/login?windowname=1", { json => '{"username":"admin","password":"initial"}' } );
is( $gwt->status, 200 );
like($gwt->content, qr%^<html>
<script type="text/javascript">
window.name='\{"token":"([0-9a-f]{40})"\}';
</script>
</html>
%, "Expected response" );

my ($token) = ($gwt->content =~ qr%token":"([0-9a-f]{40})"%);
note("token=$token");

$gwt->get_ok( "$url_prefix/rest/info?x-opsview-username=admin&x-opsview-token=$token", "Can get /rest/info with parameters via get" );
my $rest_info = $gwt->content;
note("rest_info: $rest_info");

$gwt->post( "$url_prefix/rest/info", { 'x-opsview-username' => "admin", 'x-opsview-token' => $token } );
note( "content=".$gwt->content );
is( $gwt->status, 405, "POST to /rest/info should fail" );

$gwt->post_ok( "$url_prefix/rest/info", { 'x-opsview-username' => "admin", 'x-opsview-token' => $token, 'x-tunneled-method' => 'GET' }, "Can get rest/info via POST" );
$content = $gwt->content;
is( $content, $rest_info, "Got /rest/info same as GET call" );

$gwt->post_ok( "$url_prefix/rest/config/host", { "x-opsview-username" => "admin", "x-opsview-token" => $token, "x-tunneled-method" => "PUT", json => '{"name":"opsview","alias":"fearsome shot"}' } );
$content = $gwt->content;
like( $content, qr/"alias":"fearsome shot"/, "Got changed data" ) || diag( "content=$content" );

$gwt->post_ok( "$url_prefix/rest/config/host", { "x-opsview-username" => "admin", "x-opsview-token" => $token, "x-tunneled-method" => "GET", "s.id" => 11 }, "Search for id=11" );
like( $gwt->content, qr/"rows":"1"/, "Host deleted" );

$gwt->post_ok( "$url_prefix/rest/config/host/11", { "x-opsview-username" => "admin", "x-opsview-token" => $token, "x-tunneled-method" => "DELETE" }, "Delete" );

$gwt->post_ok( "$url_prefix/rest/config/host", { "x-opsview-username" => "admin", "x-opsview-token" => $token, "x-tunneled-method" => "GET", "s.id" => 11 }, "Search for id=11" );
like( $gwt->content, qr/"rows":"0"/, "Host deleted" );

