#!/usr/bin/perl
# Test events

use warnings;
use strict;
use FindBin qw($Bin);

use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;

use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);

# Cannot add this lib before T::W::M::C
use lib "$Bin/lib";
use Opsview::TestUtils;
use Test::JSON::File;
use Test::JSON;

Opsview::Test->restore_rrds unless ( $ARGV[0] && $ARGV[0] eq "nodb" );

my $ua = Test::WWW::Mechanize::Catalyst->new;

log_in( $ua, 'admin', 'initial' );

my $rrddir = "$Bin/var/json-rrd";

$ua->get_ok('/rrdfetch?start=1242797493&end=1254121784&hsm=opsview::Opsview HTTP::size');
is_json_file( $ua->content, "$rrddir/opsview-http-size.json", "JSON output as expected" );


$ua->get_ok('/rrdfetch?start=1242797493&end=1254121784&hsm=opsview::Opsview HTTP::size&hsm=opsview::Opsview HTTP::size');
is_json_file( $ua->content, "$rrddir/opsview-http-size.json", "hsm duplicated" );

$ua->get_ok('/rrdfetch?start=1242797493&end=1254121784&hsm=opsview::Opsview HTTP::size&hsm=rubbish::service::size&hsm=opsview::Opsview HTTP::size::warning&hsm=opsview::Opsview HTTP::size::critical');
is_json_file( $ua->content, "$rrddir/opsview-http-size.json", "ignore bad hsm" );

$ua->get_ok('/rrdfetch?start=1242797493&end=1254121784&hsm=opsview::Opsview HTTP::size&hsm=opsview::Opsview HTTP::time');
is_json_file( $ua->content, "$rrddir/opsview-http-size-and-time.json", "include 2 metrics" );
my $saved_content = $ua->content;

#the following code goes to the specified link. It should get a csv and if it doesn't (in this case its called isn't). The isnt(content1,content2,"display String") compares and throws error when they are same.
$ua->get_ok('/rrdfetch?start=1242797493&end=1254121784&hsm=opsview::Opsview HTTP::size&hsm=opsview::Opsview HTTP::time&output=csv&title=graph%20title!');
isnt( $ua->content, $saved_content, "Expect different data because we ran output=csv");
is_csv_file( $ua->content, "$rrddir/opsview-http-size-and-time.csv", "CSV file contents match" );
is ($ua->response->headers->{"content-disposition"}, qq[attachment; filename="graph_title_.csv"], "Got download filename" );


$ua->get_ok('/rrdfetch?start=1242797493&end=1254121784&hsm=opsview::Opsview HTTP::time');
is_json_file( $ua->content, "$rrddir/opsview-http-time.json", "Just opsview-http-time" );

$ua->get_ok('/rrdfetch?start=1242797493&end=1254121784&hsm=opsview::Opsview HTTP::time&hsm=opsview::Opsview HTTP::time::warning&hsm=opsview::Opsview HTTP::time::critical');
is_json_file( $ua->content, "$rrddir/opsview-http-time-with-thresholds.json", "With thresholds" );

$ua->get_ok('/rrdfetch?start=1242797493&end=1254121784&hsm=opsview::TCP/IP - LAN::rta');
is_json_file( $ua->content, "$rrddir/opsview-tcpip-rta.json", "JSON for opsview::TCP/IP - LAN::rta" );

$ua->get_ok('/rrdfetch?start=1242797493&end=1254121784&keyword=cisco');
is_json_file( $ua->content, "$rrddir/opsview-keyword.json", "JSON for keyword cisco");

$ua->get_ok('/rrdfetch?start=1242797493&end=1254121784&keyword=cisco&hsm=opsview::Opsview HTTP::size');
is_json_file( $ua->content, "$rrddir/opsview-keyword.json", "JSON for keyword cisco, duplications not resent");

$ua->get_ok('/rrdfetch?start=1242797493&end=1254121784&keyword=cisco&hsm=ov-build-hardy-32::SSH::time');
is_json_file( $ua->content, "$rrddir/opsview-keyword.json", "JSON for keyword cisco, ignore bad hsm");


# Test the graph pages themselves
$ua->get_ok("/graph?host=opsview&service=TCP/IP - LAN");
$ua->content_lacks("There has been an error", "Not got an error message" );
$ua->content_contains('g.addSeries("opsview::TCP\/IP - LAN::losspct");', "Got graph information") || diag( $ua->content );
$ua->content_like( qr%<div id="infobox">.*Opsview.*UK.*Monitoring Servers.*opsview.*</div>%, "Got breadcrumbs" );


$ua->get_ok("/logout");
log_in( $ua, 'somehosts', 'somehosts' );

$ua->get_ok('/rrdfetch?start=1242797493&end=1254121784&hsm=opsview::TCP/IP - LAN::rta');
is_json_file( $ua->content, "$rrddir/opsview-tcpip-rta.json", "Same JSON for opsview::TCP/IP - LAN::rta as admin user" );

$ua->get_ok('/rrdfetch?start=1242797493&end=1254121784&hsm=opsview::Opsview HTTP::size');
is_json( $ua->content, '{"ResultSet":{"lines":[]}}', "No data here due to no permissions" );

# Set an invalid cookie
$ua->cookie_jar->{COOKIES}->{"localhost.local"}->{"/"}->{auth_tkt}->[1] = "badcookie";
$ua->get('/rrdfetch?start=1242797493&end=1254121784&hsm=opsview::TCP/IP - LAN::rta', "Set bad auth_tkt cookie");

$ua->content_like( qr/login/i, "Got login page within content" );
is( $ua->status, 401, "Got bad status code due to invalid auth cookie" );

1;
