#!/usr/bin/perl
# Test menu generation

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More;
use Test::Deep;

plan "no_plan";

use Opsview::Test qw(opsview);
use Test::WWW::Mechanize::Catalyst qw(Opsview::Web);

# Override the accessor to return test data instead
my $test_menu;
*Opsview::Web::Controller::SideNav::menus = sub {
    return $test_menu;
};

# Check module menu
my $module_menu = {
    name    => "Modules",
    entries => [
        { name => "Nagvis", url => "/nagvis" },
        { name => "MRTG",   url => "/status/network_traffic" },
        { name => "NMIS",   url => "/cgi-nmis/nmiscgi.pl", access => "ADMINACCESS" },
        {   access               => "ADMINACCESS",
            module_external_link => 1,
            name                 => "Reports",
            url                  => "http://www.opsview.com/products/enterprise-modules/reports",
        },
        {   access               => "ADMINACCESS",
            module_external_link => 1,
            name                 => "Service Desk Connector",
            url                  => "http://www.opsview.com/products/enterprise-modules/service-desk-connector",
        },
        {   access               => "ADMINACCESS",
            module_external_link => 1,
            name                 => "SMS Messaging",
            url                  => "http://www.opsview.com/products/enterprise-modules/sms-messaging",
        },
        {   access               => "ADMINACCESS",
            module_external_link => 1,
            name                 => "RANCID",
            url                  => "http://www.opsview.com/products/enterprise-modules/rancid",
        },
    ]
};

my $res;

Opsview::Web->config->{"Controller::SideNav"}->{menus} = [];
$test_menu = [];
my $c          = Opsview::Web->new;
my $controller = Opsview::Web::Controller::SideNav->new;
$res = $controller->menus_list($c);
is_deeply( $res, [$module_menu] ) || diag( Data::Dump::dump($res) );

Opsview::Web->config->{"Controller::SideNav"}->{menus} = [ { name => "stuff" }, { name => "Alerts" }, { name => "Bob" } ];
$test_menu = [ { name => "stuff" }, { name => "Alerts" }, { name => "Bob" } ];
$res = $controller->menus_list($c);
is_deeply( $res, [ { name => "stuff" }, { name => "Alerts" }, $module_menu, { name => "Bob" } ] ) || diag( Data::Dump::dump($res) );

Opsview::Web->config->{"Controller::SideNav"}->{menus} = [ { name => "Modules" }, { name => "Alerts" }, { name => "Bob" } ];
$test_menu = [ { name => "Modules" }, { name => "Alerts" }, { name => "Bob" } ];
$res = $controller->menus_list($c);
is_deeply( $res, [ $module_menu, { name => "Alerts" }, { name => "Bob" } ] ) || diag( Data::Dump::dump($res) );

1;

