#!/usr/bin/perl

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;

use Opsview::Test qw(opsview);
use Test::WWW::Mechanize;

use lib "$Bin/lib";
use Opsview::TestUtils;

my $mech = Test::WWW::Mechanize->new;

log_in( $mech, 'admin', 'initial', "http://localhost:3000" );

$mech->get_ok("/admin/servicecheck/edit/85");
$mech->content_contains('>Operations: Check Swap</option>');
$mech->content_lacks('Operations: Discards', "Ignore multiple service checks" );

$mech->get_ok("/admin/servicecheck/clone/85");
$mech->content_contains('>Operations: Check Swap</option>');

$mech->get_ok("/admin/servicecheck/new");
$mech->form_name("main_form");
$mech->content_contains('<option value="2">freshness checks</option>', "Got servicegroup popup lists" );
$mech->content_contains('>Operations: Check Swap</option>', "Dependencies list is populated" );
is( $mech->value( "stale_state" ), "3", "Default stale_state is set to UNKNOWN" );

$mech->get_ok("/admin/servicecheck/edit/87", "Got Passive set_stale service check definition");
$mech->form_name("main_form");
is( $mech->value( "stale_state" ), "0", "But a known value is set to OK" );

$mech->get_ok("/admin/servicecheck/edit/97", "Got snmp poll rate service check");
$mech->form_name("main_form");
is( $mech->value("oid"), "IF-MIB::ifInOctets.1", "Got oid" );
is( $mech->value("label"), "inOctet", "Got label" );
is( $mech->value("calculate_rate"), "per_second", "Got calculate_rate");
is( $mech->value("notification_options", 1), "w", "Got warning notify on");
is( $mech->value("notification_options", 2), "c", "Got critical notify on");
is( $mech->value("notification_options", 3), "r", "Got recovery notify on");
is( $mech->value("notification_options", 4), "u", "Got unknown notify on");
is( $mech->value("notification_options", 5), "f", "Got flapping notify on");

$mech->field("oid", "superman");
$mech->field("label", "loislane");
$mech->select("calculate_rate", "per_hour");
$mech->field("keywords", "shaggy,scooby");
$mech->field("notification_interval", "" );
$mech->select("attribute", "7" );
foreach my $i (qw/w c r u f/) {
    $mech->untick("notification_options", $i);
}
$mech->submit;

$mech->get_ok("/admin/servicecheck/edit/97", "Got new snmp poll rate service check");
$mech->form_name("main_form");
is( $mech->value("oid"), "superman", "Got new oid" );
is( $mech->value("label"), "loislane", "Got new label" );
is( $mech->value("calculate_rate"), "per_hour", "Got new calculate_rate");
is( $mech->value("keywords"), "scooby,shaggy", "Got keywords" );
is( $mech->value("notification_interval"), "", "Got blank notification interval" );
is( $mech->value("attribute"), "7", "Attribute set correctly" );
is( $mech->value("notification_options", 1), undef, "Got warning notify off");
is( $mech->value("notification_options", 2), undef, "Got critical notify off");
is( $mech->value("notification_options", 3), undef, "Got recovery notify off");
is( $mech->value("notification_options", 4), undef, "Got unknown notify off");
is( $mech->value("notification_options", 5), undef, "Got flapping notify off");

# Editing /
$mech->get_ok("/admin/servicecheck/edit/47");
$mech->form_name("main_form");
is($mech->current_form->find_input("dependencies")->{menu}->[1]->{name}, "Operations: JBoss");
$mech->select("dependencies", 34);
$mech->field("args", '-s "<script>alert(1)</script>"');
$mech->submit;

$mech->get_ok("/admin/servicecheck");
$mech->content_contains("&lt;script&gt;alert(1)&lt;/script&gt;", "<script> tags escaped");

$mech->get_ok("/admin/servicecheck/edit/47");
$mech->form_name("main_form");
is($mech->current_form->find_input("dependencies")->{menu}->[1]->{name}, "Operations: JBoss", "Saved over a submit");
is( $mech->value("servicegroup"), 1, "Servicegroup = Operations" );
$mech->field("servicegroup_new", "lexluthor" );
$mech->submit;

$mech->get_ok("/admin/servicecheck/edit/47");
$mech->form_name("main_form");
is($mech->current_form->find_input("dependencies")->{menu}->[1]->{name}, "Choose servicechecks from list", "No save since not selected" );
is( $mech->value("servicegroup"), 3, "Service group changed" );

$mech->get_ok("/admin/servicecheck/edit/47");
$mech->form_name("main_form");
$mech->field("servicegroup_new", "bad!chars");
$mech->submit;
is( $mech->status, 500, "Error with new service group name" );
$mech->content_contains("name: Invalid", "Got error with service group creation" );

# Create snmptrap check
$mech->get_ok("/admin/servicecheck/new");
$mech->form_name("main_form");
$mech->field("name", "snmptraptest");
$mech->field("checktype", 4);
$mech->submit;

$mech->content_contains("/admin/snmptraprule/", "Found rules page next" ) || diag($mech->content);

# Clone snmptraprule
$mech->get_ok("/admin/servicecheck/clone/81");
$mech->form_name("main_form");
$mech->field("name", "snmptrapclone");
$mech->submit;

$mech->content_contains("/admin/snmptraprule/", "Found rules page next" ) || diag($mech->content);
my $content = $mech->content;
my ($url) = ($content =~ m%URL=http://localhost:3000(.*)"% );
$mech->get_ok( $url );

my $rows = grep /Edit /, split( "\n", $mech->content );
is( $rows, 1, "And have got the 1 rule cloned from previous service check" );

$mech->get_ok("/admin/servicecheck/clone/95", "Checking cloning of Interface service check");
$mech->content_contains("Checks for interfaces", "Is right clone page" );
#$mech->get_ok("/admin/servicecheck/clone/2", "Checking cloning of Interface service check");
$mech->form_name("main_form");
$mech->field("name", "DifferentInterface");
$mech->submit;
$mech->content_contains('<meta http-equiv="refresh" content="0; URL=http://localhost:3000/admin/snmptraprule/list?servicecheck=', "Got expected redirect following successful submission") || diag( $mech->content );
$mech->get_ok("/admin/servicecheck?q=DifferentInt", "Getting a list");
$mech->content_contains("DifferentInterface", "Got name of new service check");
