#!perl

package Opsview::Test::WWW::Mechanize;
use Test::WWW::Mechanize;
use base Test::WWW::Mechanize;

my $redirect_ok=1;
sub set_redirect_ok { shift; $redirect_ok=shift }
sub redirect_ok {
    my $self = shift;
    if ($redirect_ok) {
        return $self->SUPER::redirect_ok(@_);
    } else {
        return 0;
    }
}

package main;
use warnings;
use strict;

use FindBin qw($Bin);
use lib "$Bin/lib";
use lib "/usr/local/nagios/lib";

use Test::WWW::Mechanize;
use HTTP::Cookies;
use Sys::Hostname;

use Opsview::Web::Test;

use Opsview::Schema;
use Opsview::Config;

use Test::More qw(no_plan);
use Test::Deep;

my $rs = Opsview::Schema->my_connect->resultset("Contacts");

my $ua = Opsview::Test::WWW::Mechanize->new( autocheck => 0 );
my $cj = HTTP::Cookies->new();
$ua->cookie_jar($cj);

my $hostname = Sys::Hostname::hostname;
$hostname = "localhost:3000";

# No cookies here - should get login page
my $res;


$res = $ua->get("http://$hostname/");
ok( $res->is_success, "Got successful response" );
is( $res->code, 200, "Status 200" );
like( $res->content, "/login_username/s", "Returned login_username, so is a login page" );
my $content = $res->content;
$content =~ s%<noscript>.*?</noscript>%%s;    # Remove "JavaScript is required message"
unlike( $content, "/<h3>/s", "Check that no login error is displayed" );

cmp_deeply( $cj->{COOKIES}, {}, "Cookie jar is empty" );

$ua->set_redirect_ok(0);
$ua->field( 'login_username', 'admin' );
$ua->field( 'login_password', 'initial' );
$ua->field( "noscript" => 0 );
$ua->current_form->action;
$ua->submit;

# Check IE8 header is set to allow iframes in cross domain
is( $ua->response->header('P3P'), 'CP="IDC DSP COR ADM DEVi TAIi PSA PSD IVAi IVDi CONi HIS OUR IND CNT"', "Check that IE8 header is set");

