#!perl

use warnings;
use strict;

use FindBin qw($Bin);
use lib "$Bin/lib";
use lib "/usr/local/nagios/lib";

use WWW::Mechanize;
use HTTP::Cookies;
use Sys::Hostname;

use Opsview::Web::Test;

use Opsview;
use Opsview::Contact;

use Test::More qw(no_plan);
use Test::Deep;

my $ua = WWW::Mechanize->new( autocheck => 0 );
my $cj = HTTP::Cookies->new();
$ua->cookie_jar($cj);

my $hostname = Sys::Hostname::hostname;
$hostname = "localhost:3000";

# No cookies here - should get login page
my $res;
$res = $ua->get("http://$hostname/");
ok( $res->is_success, "Got successful response" );
is( $res->code, 200, "Status 200" );
like( $res->content, "/login_username/s", "Returned login_username, so is a login page" );
my $content = $res->content;
$content =~ s%<noscript>.*?</noscript>%%s;    # Remove "JavaScript is required message"
unlike( $content, "/<h3>/s", "Check that no login error is displayed" );

cmp_deeply( $cj->{COOKIES}, {}, "Cookie jar is empty" );

$ua->field( 'login_username', 'admin' );
$ua->field( 'login_password', 'initial' );
$ua->field( "noscript" => 0 );
$ua->current_form->action;
$ua->submit;

ok( $ua->res->is_success, "Successful reply" );
like( $ua->content, "/Host Group Summary &gt; Opsview/", "Found heading for HH page" );

# Now goto a public page - this doesn't check the auth_tkt
Opsview::Contact->search( name => "admin" )->delete_all;
$res = $ua->get("http://$hostname/about");
ok( $res->is_success, "Successful reply" );
like( $res->content, "/Opsview is developed by/s", "Got public page" );

# Add this header to simulate Prototype's call
# Should get access denied because no public access to this
# Update: Get 200
$ua->add_header( "x-requested-with" => "XMLHttpRequest" );
$res = $ua->get("http://$hostname/search/host?q=cis");
is( $res->code, 200, "Get OK status code but ..." );
is( $res->content, "AJAX error", "... ajax error as content" );

# Check logout page doesn't give an error
$ua->delete_header("x-requested-with");
$res = $ua->get("http://$hostname/logout");
ok( $res->is_success, "Check logout page doesn't give error" );
like( $res->content, "/login_password/", "Got login page" );
