#!perl
# Needs to follow straight after 900-backup as that sets up the appropriate objects and state

use warnings;
use strict;

use utf8;
use Test::WWW::Mechanize;
use HTTP::Cookies;
use Sys::Hostname;

use Test::More qw(no_plan);
use Test::Deep;

my $ua = Test::WWW::Mechanize->new();
my $cj = HTTP::Cookies->new();
$ua->cookie_jar($cj);

my $hostname = Sys::Hostname::hostname;
$hostname = "localhost:3000";

# No cookies here - should get login page
my $res;
$res = $ua->get("http://$hostname/");
ok( $res->is_success, "Got successful response" );
is( $res->code, 200, "Status 200" );
like( $res->content, "/login_username/s", "Returned login_username, so is a login page" );

cmp_deeply( $cj->{COOKIES}, {}, "Cookie jar is empty" );

$ua->field( 'login_username', 'admin' );
$ua->field( 'login_password', 'initial' );
$ua->field( "noscript" => 0 );
$ua->current_form->action;
$ua->submit;

ok( $ua->res->is_success, "Successful reply" );
like( $ua->content, "/Host Group Summary &gt; Opsview/", "Found heading for HH page" );

# Check POST problem with Nagios CGIs
# Nagios CGIs cannot read both GET query parameters and POST parameters together
{
    my $req = HTTP::Request->new(
        "POST" => "http://$hostname/cgi-bin/status.cgi",
        [qw(Content-Type application/x-www-form-urlencoded)],
        "host=opsview",
    );
    ok( my $res = $ua->request($req), "Request" );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );

    like( $res->content, "/Service Status Details For Host/", "Got Nagios CGIs for status" );
    unlike( $res->content, "/opsviewdev1/", "...without opsviewdev1 information as requested just opsview host" );
}

$ua->get_ok("/cgi-bin/extinfo.cgi?type=1&host=opsview");
$ua->follow_link_ok( { text => "Re-schedule the next check of this host" } );
$ua->form_with_fields("force_check");
is($ua->value("force_check"), undef, "force_check not set - otherwise could cause problems with distributed setup" );

$ua->back;
$ua->follow_link_ok( { text => "Schedule downtime for this host" } );
isnt($ua->form_name("main_form"), undef, "Is an Opsview downtime form" );

$ua->back;
$ua->follow_link_ok( { text => "Schedule downtime for all services on this host" } );
isnt($ua->form_name("main_form"), undef, "Is an Opsview downtime form" );


$ua->get_ok("/cgi-bin/status.cgi?host=賢敦");
$ua->content_contains('Host: 賢敦', "Check that the UTF8 name is returned in the page header");
$ua->content_contains(qq{Service Status Details For Host '賢敦'}, "Check that the UTF8 name is returned in the cgis");

$ua->get_ok("/cgi-bin/extinfo.cgi?type=2&host=opsview&service=MySQL");
$ua->follow_link_ok( { text => "Re-schedule the next check of this service" } );
$ua->form_with_fields("force_check");
is($ua->value("force_check"), undef, "force_check not set - otherwise could cause problems with distributed setup" );

1;
